def coarse_inv_sqrt(x: float) -> float:
    """
    Coarse inverse square root. (Implementation from Wikipedia)
    """
    ...
    
def fined_inv_sqrt(x: float) -> float:
    """
    Fined inverse square root. (Implementation from math.h)
    """
    ...